add_relation_definition('Activity', 'associated_with', 'CWUser')
rql('SET A associated_with U WHERE A associated_with AG, AG agent_user U')
commit()
drop_relation_definition('Activity', 'associated_with', 'Agent')

sync_schema_props_perms(('CWUser', 'authority', 'Organization'))

add_relation_definition(
    'OrganizationUnit', 'use_authorityrecord', 'AuthorityRecord')

# Set default "use_authorityrecord" relation to existing AuthorityRecord
with cnx.deny_all_hooks_but():
    rql('SET OU use_authorityrecord X WHERE OU archival_role AR, AR name "archival"')
    commit(ask_confirm=False)

add_attribute('Organization', 'ark')

drop_cube('pyramid')

schema_wf = get_workflow_for('ConceptScheme')
with cnx.deny_all_hooks_but():
    rql('SET X in_state S WHERE X is ConceptScheme, S eid %(s)s, NOT X in_state Y',
        {'s': schema_wf.state_by_name('published').eid})
commit()


try:
    sql('CREATE UNIQUE INDEX words_unique_idx ON words (etype, word)')
    sql('CREATE INDEX words_word_idx ON words USING gin(word gin_trgm_ops)')
    commit(ask_confirm=False)
except Exception:
    rollback()



for etype in ('Agent', 'OrganizationUnit'):
    wf = get_workflow_for(etype)
    wf.transition_by_name('publish').cw_delete()
    wf.state_by_name('draft').cw_delete()
    wf.cw_set(initial_state=wf.state_by_name('published'))
    cnx.execute('DELETE TrInfo X WHERE NOT X from_state S')
    cnx.commit()



# coding: utf-8
# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref schema"""

from yams.buildobjs import EntityType, RelationDefinition, String, Int, ComputedRelation
from yams.constraints import UniqueConstraint

from cubicweb import _
from cubicweb.schema import (RO_ATTR_PERMS, ERQLExpression, RRQLExpression,
                             RQLConstraint, RQLVocabularyConstraint, WorkflowableEntityType,
                             make_workflowable)
from cubicweb.schemas.base import ExternalUri, EmailAddress

from cubicweb_skos import schema as skos
from cubicweb_prov import schema as prov
from cubicweb_eac import schema as eac
from cubicweb_seda.schema import simplified_profile
from cubicweb_seda.schema.seda2 import SEDAArchiveTransfer
from cubicweb_compound import utils


def publication_permissions(cls, groups=('managers', 'users')):
    """Set __permissions__ of `cls` entity type class preventing modification
    when not in state "draft".
    """
    groups = ', '.join('"{}"'.format(group) for group in groups)
    cls.__permissions__ = cls.__permissions__.copy()
    cls.__permissions__['update'] = (
        ERQLExpression('U in_group G, G name IN ({}), '
                       'X in_state ST, ST name "draft"'.format(groups)),
    )
    cls.__permissions__['delete'] = (
        ERQLExpression('U in_group G, G name IN ({}), '
                       'X in_state ST, ST name "draft"'.format(groups)),
    )
    return cls


def authority_permissions_etype(cls):
    """Set __permissions__ of `cls` entity type class to ensure user can
    update/delete provided its authority is the same as the entity's authority.

    Creation permission is ensured by permission of the authority relation.
    """
    cls.__permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', 'users'),
        'update': ('managers', ERQLExpression('U authority A, X authority A')),
        'delete': ('managers', ERQLExpression('U authority A, X authority A')),
    }
    return cls


def authority_permissions_rdef(cls):
    """Set __permissions__ of `cls` relation definition class to ensure user can
    create/delete provided its authority is the same as the relation subject's
    authority.
    """
    cls.__permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', RRQLExpression('U authority A, S authority A')),
        'delete': ('managers', RRQLExpression('U authority A, S authority A')),
    }
    return cls


def groups_permissions(cls):
    """Set __permissions__ of `cls` entity type class preventing modification
    when user is not in managers or users group.
    """
    cls.__permissions__ = cls.__permissions__.copy()
    cls.__permissions__['update'] = (
        ERQLExpression('U in_group G, G name IN ("managers", "users")', 'U'),
    )
    return cls


# Disable "update" for ExternalUri as these should only come from imported data
# and are meant to only be created or deleted.
ExternalUri.__permissions__ = ExternalUri.__permissions__.copy()
ExternalUri.__permissions__['update'] = ()


# Customization of EmailAddress entity type.
EmailAddress.remove_relation('alias')


# Customization of eac schema.
make_workflowable(eac.AuthorityRecord)
groups_permissions(eac.AuthorityRecord)

eac.agent_kind.__permissions__ = {
    'read': ('managers', 'users', 'guests'),
    'add': ('managers', RRQLExpression('U has_update_permission S')),
    'delete': ('managers', RRQLExpression('U has_update_permission S')),
}
eac.agent_kind.constraints = [
    RQLConstraint('NOT EXISTS(Z authority_record S)'
                  ' OR '
                  'EXISTS(A authority_record S, A is Agent, '
                  '       O name "person")'
                  ' OR '
                  'EXISTS(OU authority_record S, OU is IN (Organization, OrganizationUnit), '
                  '       O name "authority")',
                  msg=_('This record is used by a relation forbidding to change its type')),
]

for etype_def in (eac.ChronologicalRelation,
                  eac.HierarchicalRelation,
                  eac.AssociationRelation):
    etype_def.__permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', 'users'),
        'update': ('managers', 'users'),
        'delete': ('managers', 'users'),
    }


# SKOS #########################################################################

make_workflowable(skos.ConceptScheme)
publication_permissions(skos.ConceptScheme, groups=('managers',))
skos.ConceptScheme.__permissions__['add'] = ('managers',)

skos.in_scheme.cardinality = '1*'


class Organization(EntityType):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', ),
        'update': ('managers', ),
        'delete': ('managers', ),
    }
    name = String(required=True, fulltextindexed=True, unique=True)


@authority_permissions_etype
class OrganizationUnit(WorkflowableEntityType):
    __unique_together__ = [('name', 'authority')]
    name = String(required=True, fulltextindexed=True)


@authority_permissions_etype
class Agent(WorkflowableEntityType):
    __unique_together__ = [('name', 'authority')]
    name = String(required=True, fulltextindexed=True)


class user_authority(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers',),
        'delete': (),
    }
    name = 'authority'
    subject = 'CWUser'
    object = 'Organization'
    cardinality = '?*'
    inlined = True
    constraints = [
        RQLConstraint('NOT EXISTS(A agent_user S) '
                      'OR EXISTS(B agent_user S, B authority O)'),
    ]


class others_authority(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', RRQLExpression('U authority O'),),
        'delete': (),
    }
    name = 'authority'
    subject = ('OrganizationUnit', 'Agent')
    object = 'Organization'
    cardinality = '1*'
    composite = 'object'
    inlined = True


class agent_user(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers',),
        'delete': ('managers',),
    }
    subject = 'Agent'
    object = 'CWUser'
    cardinality = '??'
    inlined = True
    description = _('the application user related to this agent')
    constraints = [
        RQLConstraint('NOT EXISTS(O authority A) '
                      'OR EXISTS(O authority B, S authority B)'),
    ]


class _authority_record(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', RRQLExpression('U has_update_permission S')),
        'delete': ('managers',),
    }
    name = 'authority_record'
    object = 'AuthorityRecord'
    cardinality = '??'
    inlined = True
    description = _('the authority record describing this agent')


class ou_authority_record(_authority_record):
    subject = 'OrganizationUnit'
    description = _('the authority record describing this organization unit')
    constraints = [
        RQLConstraint('O agent_kind K, K name "authority"'),
    ]


class o_authority_record(ou_authority_record):
    subject = 'Organization'
    description = _('the authority record describing this organization')


class agent_authority_record(_authority_record):
    subject = 'Agent'
    description = _('the authority record describing this agent')
    constraints = [
        RQLConstraint('O agent_kind K, K name "person"'),
    ]


class contact_point(RelationDefinition):
    subject = 'OrganizationUnit'
    object = 'Agent'
    cardinality = '?*'
    inlined = True
    constraints = [
        RQLConstraint('S authority A, O authority A'),
        RQLVocabularyConstraint('O in_state ST, ST name "published"'),
    ]
    description = _('set an agent as the contact point of an organization unit')


class archival_unit(RelationDefinition):
    subject = 'Organization'
    object = 'OrganizationUnit'
    cardinality = '?*'
    description = _("the archival unit responsible for dealing with the organization's "
                    "documents")
    constraints = [RQLConstraint('O archival_role AR, AR name "archival"'),
                   RQLVocabularyConstraint('O in_state ST, ST name "published"')]


class archival_authority(ComputedRelation):
    rule = 'S archival_unit OU, OU authority O'


class use_authorityrecord(RelationDefinition):
    subject = 'OrganizationUnit'
    object = 'AuthorityRecord'
    cardinality = '**'
    description = _("authority records used by this archival unit")
    constraints = [
        RQLConstraint('S archival_role AR, AR name "archival"'),
    ]


class ArchivalRole(EntityType):
    """An archival role determines the kind of action (e.g. deposit or control)
    an agent may perform on an archive entity.
    """
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', ),
        'update': ('managers', ),
        'delete': ('managers', ),
    }
    name = String(required=True, unique=True, internationalizable=True)


class archival_role(RelationDefinition):
    subject = 'OrganizationUnit'
    object = 'ArchivalRole'
    cardinality = '**'
    description = _("the organization unit's archival role (producer, control, etc.)")


class use_email(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests',),
        'add': ('managers', RRQLExpression('U has_update_permission S'),),
        'delete': ('managers', RRQLExpression('U has_update_permission S'),),
    }
    subject = 'Agent'
    object = 'EmailAddress'
    cardinality = '*?'
    composite = 'subject'
    fulltext_container = 'subject'


class phone_number(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests',),
        'add': ('managers', RRQLExpression('U has_update_permission S'),),
        'delete': ('managers', RRQLExpression('U has_update_permission S'),),
    }
    subject = 'Agent'
    object = 'PhoneNumber'
    cardinality = '*1'
    composite = 'subject'


@authority_permissions_rdef
class related_concept_scheme(RelationDefinition):
    subject = 'OrganizationUnit'
    object = 'ConceptScheme'
    cardinality = '**'
    description = _('concept schemes used by the agent')


# PROV #########################################################################

# we have to let addition permission to managers/users but eac use them for user-land imports but
# empty permission would be better because those are expected to be set by hooks. We could probably
# change eac import to by-pass security for activities?

class generated(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': (),
        'delete': (),
    }
    subject = 'Activity'
    object = ('Concept', 'ConceptScheme', 'SEDAArchiveTransfer')


class used(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': (),
        'delete': (),
    }
    subject = 'Activity'
    object = ('Concept', 'ConceptScheme', 'SEDAArchiveTransfer')


class associated_with(RelationDefinition):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': (),
        'delete': (),
    }
    subject = 'Activity'
    object = 'CWUser'


prov.Activity.__permissions__ = {
    'read': ('managers', 'users', 'guests'),
    'add': ('managers', 'users'),
    'update': (),
    'delete': (),
}

eac.used.__permissions__ = eac.generated.__permissions__ = {
    'read': ('managers', 'users', 'guests'),
    'add': (RRQLExpression('U has_update_permission O'),),
    'delete': (),
}


# ARK ##########################################################################

class ark(RelationDefinition):
    __permissions__ = RO_ATTR_PERMS
    subject = (
        'Agent',
        'AuthorityRecord',
        'Concept',
        'ConceptScheme',
        'Organization',
        'OrganizationUnit',
        'SEDAArchiveTransfer',
    )
    object = 'String'
    description = _('ARK Identifier - will be generated if not specified')
    constraints = [UniqueConstraint()]
    cardinality = '11'


class ArkNameAssigningAuthority(EntityType):
    """Name Assigning Authority (NAA) for ARK generation."""
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', ),
        'update': ('managers', ),
        'delete': ('managers', ),
    }
    who = String(required=True, unique=True,
                 description=_('official organization name'))
    what = Int(required=True, unique=True,
               description=_('Name Assigning Authority Number (NAAN)'))


class _ark_naa(RelationDefinition):
    name = 'ark_naa'
    object = 'ArkNameAssigningAuthority'
    inlined = True


class organization_ark_naa(_ark_naa):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', ),
        'delete': ('managers', ),
    }
    subject = 'Organization'
    cardinality = '?*'
    description = _("ARK identifier Name Assigning Authority (NAA) - "
                    "you'll need one to start creating objects")


class mandatory_ark_naa(_ark_naa):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', 'users',),
        'delete': (),
    }
    subject = ('AuthorityRecord', 'SEDAArchiveTransfer')
    cardinality = '1*'
    description = _("ARK identifier Name Assigning Authority (NAA)")


class optional_ark_naa(_ark_naa):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers', 'users',),
        'delete': (),
    }
    subject = ('ConceptScheme', 'SKOSSource')
    cardinality = '?*'
    description = _("ARK identifier Name Assigning Authority (NAA), "
                    "necessary to create ARK for concepts which don't have one yet.")


# SEDA #######################################################################

make_workflowable(SEDAArchiveTransfer)
publication_permissions(SEDAArchiveTransfer)


simplified_profile.default = True


@authority_permissions_rdef
class use_profile(RelationDefinition):
    subject = 'OrganizationUnit'
    object = 'SEDAArchiveTransfer'
    cardinality = '**'
    constraints = [RQLConstraint('S archival_role R, R name "deposit"'),
                   RQLConstraint('O in_state ST, ST name "published"')]


class new_version_of(RelationDefinition):
    __permissions__ = {'read': ('managers', 'users', 'guests'),
                       'add': ('managers', 'users',
                               RRQLExpression('O in_state ST, ST name "published"')),
                       'delete': ()}
    subject = 'SEDAArchiveTransfer'
    object = 'SEDAArchiveTransfer'
    cardinality = '??'
    inlined = True


def post_build_callback(schema):
    # Make "in_scheme" relation type inlined so that it gets considered
    # directly (as attributes) during dataimport to help ARK identifier
    # generation.
    schema['in_scheme'].inlined = True

    schema['simplified_profile'].rdefs['SEDAArchiveTransfer', 'Boolean'].default = True  # XXX
    from . import ConceptSchemeGraph
    graph = ConceptSchemeGraph(schema)
    utils.graph_set_etypes_update_permissions(schema, graph, 'ConceptScheme')
    utils.graph_set_write_rdefs_permissions(schema, graph, 'ConceptScheme')

    # permissions override
    schema['Label'].set_action_permissions('delete', ('managers', 'users'))
    for rtype in ('in_scheme', 'broader_concept', 'label_of'):
        for rdef in schema[rtype].rdefs.values():
            rdef.set_action_permissions('add', ('managers', 'users'))
            if rtype == 'label_of':
                rdef.set_action_permissions('delete', ('managers', 'users'))

    # reset permissions, unexpectedly changed by seda's post_build_callback
    def sync_perms(erdef, permissions):
        for action, permissions in permissions.items():
            erdef.set_action_permissions(action, permissions)

    sync_perms(schema['Activity'], prov.Activity.__permissions__)
    for rdef_def in (generated, used, associated_with):
        objtypes = rdef_def.object if isinstance(rdef_def.object, tuple) else (rdef_def.object,)
        for objtype in objtypes:
            rdef = schema[rdef_def.__name__].rdefs[(rdef_def.subject, objtype)]
            sync_perms(rdef, rdef_def.__permissions__)



# coding: utf-8
# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref postcreate script"""

from cubicweb import _
from cubicweb_saem_ref import workflows


set_property('ui.site-title', u'Référentiel SAEM')
set_property('ui.language', u'fr')
set_property('ui.date-format', u'%d/%m/%Y')
set_property('ui.datetime-format', u'%d/%m/%Y %H:%M')

for name in [_('producer'), _('deposit'), _('archival'), _('control'),
             _('enquirer'), _('seda-actor')]:
    create_entity('ArchivalRole', name=name)

for etype in ('Agent', 'AuthorityRecord', 'ConceptScheme', 'OrganizationUnit',
              'SEDAArchiveTransfer'):
    workflows.define_publication_workflow(add_workflow, etype)
commit()

schema_wf = get_workflow_for('ConceptScheme')
with cnx.deny_all_hooks_but():
    rql('SET X in_state S WHERE X is ConceptScheme, S eid %(s)s',
        {'s': schema_wf.state_by_name('published').eid})
commit()



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref startup views"""

from contextlib import contextmanager

from cubicweb import tags
from cubicweb.web.views import baseviews, startup, uicfg

from . import add_etype_link, import_etype_link


for etype in ('AgentKind', 'ArchivalRole', 'AgentPlace',
              'AssociationRelation', 'ChronologicalRelation', 'HierarchicalRelation',
              'EACResourceRelation', 'Activity', 'SEDADate'):
    uicfg.indexview_etype_section[etype] = 'subobject'


def view_link(req, *args, **kwargs):
    """Return an HTML link with a "view" icon."""
    return tags.a(u'', href=req.build_url(*args, **kwargs),
                  klass='icon-eye pull-right',
                  title=req._('View'))


@contextmanager
def index_box(w, title_html, panel_class='panel-info'):
    w(u'<div class="{}">'.format('col-md-4' if panel_class == 'panel-info' else ''))
    w(u' <div class="panel {}">'.format(panel_class))
    w(u'  <div class="panel-heading panel-title">{0}</div>'.format(title_html))
    yield
    w(u' </div>')
    w(u'</div>')


class ListGroupView(baseviews.ListView):
    """Same a 'list' view but with a 'list-group' CSS class (for list items in
    particular).
    """
    __regid__ = 'listgroup'

    def call(self, title=None, subvid=None, listid=None, **kwargs):
        return super(ListGroupView, self).call(
            klass='list-group', title=title, subvid=subvid, listid=listid, **kwargs)

    def cell_call(self, row, col=0, vid=None, klass=None, **kwargs):
        self.w(u'<li class="list-group-item">')
        self.wview(self.item_vid, self.cw_rset, row=row, col=col, vid=vid, **kwargs)
        self._view_modification_date(row, col)
        self.w(u'</li>\n')

    def _view_modification_date(self, row, col):
        _ = self._cw._
        entity = self.cw_rset.get_entity(row, col)
        self.w(u'<div class="text-muted">')
        self.w(u'<span>%s</span> ' % _('latest update on'))
        self.w(u'<span class="value">%s</span> '
               % self._cw.format_date(entity.modification_date))
        self.w(u'</div>')


class SAEMRefIndexView(startup.IndexView):
    """Index view for SAEM-Ref"""
    etype_rql = {
        'AuthorityRecord': (
            'Any X,XN,XA,MD ORDERBY MD DESC LIMIT 5 WHERE X is AuthorityRecord, '
            'X modification_date MD, X ark XA, '
            'NE name_entry_for X, NE form_variant "authorized", NE parts XN'),
        'ConceptScheme': (
            'Any X,XT,XA,XD,XDF,MD ORDERBY MD DESC LIMIT 5 WHERE X is ConceptScheme, '
            'X modification_date MD, X title XT, X ark XA, '
            'X description XD, X description_format XDF'),
        'SEDAArchiveTransfer': (
            'Any X,XT,XA,XD,MD ORDERBY MD DESC LIMIT 5 WHERE X is SEDAArchiveTransfer, '
            'X modification_date MD, X title XT, X ark XA, '
            'X user_annotation XD'),
        'Organization': (
            'Any X,XN,XA,MD ORDERBY MD DESC LIMIT 5 WHERE X is Organization, '
            'X modification_date MD, X name XN, X ark XA'),
    }

    def _render_modified_rset(self, rset):
        if rset:
            self.w(tags.div(self._cw._('recently modified entities'),
                            klass='panel-body text-mutted'))
            self._cw.view('listgroup', rset=rset, w=self.w)
        else:
            self.w(tags.div(self._cw._('no entity'),
                            klass='panel-body text-mutted'))

    def render_box(self, etype, import_url=None):
        req = self._cw
        title_html = u'<b>{0}</b>'.format(req.__(etype + '_plural'))
        # `view`, `add`, `import` buttons inserted in reversed order because of 'pull-right'
        if import_url:
            title_html += import_etype_link(req, etype, import_url)
        title_html += add_etype_link(req, etype)
        rset = req.execute(self.etype_rql[etype])
        if rset:
            title_html += view_link(req, etype)
        with index_box(self.w, title_html):
            self._render_modified_rset(rset)

    def render_seda_lib_box(self):
        req = self._cw
        title_html = u'<b>{0}</b>'.format(req._('SEDA components'))
        title_html += add_etype_link(req, 'SEDAArchiveUnit', unit_type='unit_content')
        rset = req.execute(
            'Any X,MD ORDERBY MD DESC LIMIT 5 '
            'WHERE X is SEDAArchiveUnit, X modification_date MD, '
            'NOT X seda_archive_unit P')
        if rset:
            title_html += view_link(req, 'sedalib')
        with index_box(self.w, title_html):
            self._render_modified_rset(rset)

    def entities(self):
        self.w(u'<div class="row">')
        for etype, import_url in [
            ('Organization', None),
            ('AuthorityRecord', self._cw.build_url('view', vid='eac.import')),
            ('ConceptScheme', self._cw.build_url('add/skossource')),
            ('SEDAArchiveTransfer', None),
        ]:
            self.render_box(etype, import_url)
        self.render_seda_lib_box()
        self.w(u'</div>')


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__,
                      butclasses=(SAEMRefIndexView, ))
    vreg.register_and_replace(SAEMRefIndexView, startup.IndexView)



# coding: utf-8
# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref test for pyramid views."""

import lxml.etree

from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.pyramid.test import PyramidCWTest

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class EACWSTC(PyramidCWTest):
    configcls = PostgresApptestConfiguration
    settings = {'cubicweb.bwcompat': True}

    def test_xml_post(self):
        with self.admin_access.client_cnx() as cnx:
            testutils.authority_record(cnx, u'TEST AR')
            cnx.commit()
        with open(self.datapath('D33-100.xml')) as stream:
            data = stream.read()

        res = self.webapp.post('/authorityrecord',
                               params=data,
                               content_type='application/xml',
                               headers={'Accept': 'application/json'},
                               status=401)
        self.assertEqual(res.json,
                         {'error': 'This service requires authentication.'})

        self.login()
        res = self.webapp.post('/authorityrecord',
                               params=data,
                               content_type='application/xml',
                               headers={'Accept': 'application/json'},
                               status=403)
        self.assertEqual(res.json,
                         {'error': 'Authenticated user is not linked to an organisation, '
                          'or his organisation has no ARK naming authority.'})

        # to fix previous error, link user to an organization with a ARK NAA
        with self.admin_access.client_cnx() as cnx:
            org = testutils.authority_with_naa(cnx)
            cnx.user.cw_set(authority=org)
            cnx.commit()

        res = self.webapp.post('/authorityrecord',
                               params='not xml',
                               content_type='application/xml',
                               headers={'Accept': 'application/json'},
                               status=400)

        expected_details = "Start tag expected, '<' not found, line 1, column 1"
        if lxml.etree.LXML_VERSION[:2] >= (3, 7):
            expected_details += " (line 1)"
        self.assertEqual(res.json,
                         {'error': u'Invalid XML file',
                          'details': expected_details})

        res = self.webapp.post('/authorityrecord',
                               params='<xml xmlns="bla"/>',
                               content_type='application/xml',
                               headers={'Accept': 'application/json'},
                               status=400)
        self.assertEqual(res.json,
                         {'error': 'Unexpected EAC data',
                          'details': 'Missing tag control in XML file'})

        res = self.webapp.post('/authorityrecord',
                               params=data,
                               content_type='application/xml',
                               headers={'Accept': 'application/json'})
        self.assertEqual(list(res.json), ['ark'])

    def test_default_route(self):
        with self.admin_access.client_cnx() as cnx:
            testutils.authority_record(cnx, u'TEST AR')
            cnx.commit()
        res = self.webapp.get('/authorityrecord')
        self.assertEqual(res.content_type, 'text/html')
        # XXX it would be nice if POST with other Content-Type would be routed to cubicweb. See
        # attempt to set 'header' route predicate in pviews.
        # res = self.webapp.post('/authorityrecord')
        # self.assertEqual(res.content_type, 'text/html')



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref entity's classes and adapters for SEDA classes"""

from logilab.common.decorators import monkeypatch

from cubicweb.predicates import is_instance, is_in_state

from cubes.oaipmh import MetadataFormat
from cubes.oaipmh.entities import OAISetSpec
from cubicweb_seda.entities import SEDAArchiveTransferIClonableAdapter
from cubicweb_seda.entities.custom import SEDAArchiveTransfer
from cubicweb_seda.entities.profile_generation import SEDA2ExportAdapter, SEDA1XSDExport
from cubicweb_seda.views import export

from .. import cwuri_url
from . import oai


# monkeypatch cwuri_url method instead of subclassing and replacing adapters because there are 5 of
# them impacted (SEDA 2 rng, 1/0.2 rng/xsd) while we may change it once for all on the common base
# class
SEDA2ExportAdapter.cwuri_url = staticmethod(cwuri_url)


# hijack xsd_archival_agreement to insert profile as well. Use a monkey-patch rather than new
# implementation to avoid doing it twice, once for SEDA 1 another for SEDA 0.2

orig_xsd_archival_agreement = SEDA1XSDExport.xsd_archival_agreement


@monkeypatch(SEDA1XSDExport)
def xsd_archival_agreement(self, parent, transfer):
    orig_xsd_archival_agreement(self, parent, transfer)
    self.element_schema(parent, 'ArchivalProfile', 'qdt:ArchivesIDType',
                        fixed_value=u'ark:/' + transfer.ark)


# override agency_id to return ark instead of eid
@monkeypatch(SEDA1XSDExport)
def agency_id(self, agency):
    return None if agency.agency is None else 'ark:/' + agency.agency.ark


orig_xsd_transferring_agency_archive_identifier = \
    SEDA1XSDExport.xsd_transferring_agency_archive_identifier


@monkeypatch(SEDA1XSDExport)
def xsd_transferring_agency_archive_identifier(self, parent, content_entity, tag_name):
    if content_entity.transferring_agency_archive_unit_identifier:
        # XXX should not happen since we hide the field (views/seda.py).
        return orig_xsd_transferring_agency_archive_identifier(
            self, parent, content_entity, tag_name)
    # Insert an element with a "free" value (no value) that should be
    # overridden by downstream consumers.
    self.element_schema(
        parent, tag_name, 'qdt:ArchiveIDType',
        xsd_attributes=self.xsd_attributes_scheme())


class SAEMSEDAArchiveTransferIClonableAdapter(SEDAArchiveTransferIClonableAdapter):
    __select__ = SEDAArchiveTransferIClonableAdapter.__select__ & is_in_state('published')
    rtype = 'new_version_of'
    skiprtypes = SEDAArchiveTransferIClonableAdapter.skiprtypes + ('clone_of', 'used', 'generated')


class SEDAArchiveTransfer(SEDAArchiveTransfer):
    SEDAArchiveTransfer.fetch_attrs.append('ark')

    def __getstate__(self):
        # Exclude ark from copy to have so that it can be generated for the clone
        odict = self.__dict__.copy()
        odict['cw_attr_cache'].pop('ark', None)
        return odict

    @property
    def predecessor(self):
        """The predecessor of the current profile, that is the one that has been replaced by it"""
        predecessors = self.new_version_of
        if predecessors:
            return predecessors[0]
        return None

    @property
    def successor(self):
        """The successor of the current profile, that is the one that has replaced it"""
        successors = self.reverse_new_version_of
        if successors:
            return successors[0]
        return None


class TransferringAgentOAISetSpec(OAISetSpec):
    """OAI-PMH set specifier to match SEDAArchiveTransfer related to a transferring
    agent.
    """

    def setspecs(self, cnx):
        rset = cnx.execute(
            'Any A WHERE X archival_role R, R name "deposit", X ark A')
        desc = 'SEDA Profile used by organization {0}'
        for value, in rset.rows:
            yield value, desc.format(value)

    def setspec_restrictions(self, value):
        qs = ('A use_profile X, A archival_role R'
              ', R name "deposit", A ark %(value)s'
              ', X in_state ST, NOT ST name "draft"')
        return qs, {'value': value}


PROFILE_METADATA_FORMATS = {
    'seda02xsd': (
        MetadataFormat(
            schema='https://www.w3.org/2001/XMLSchema.xsd',
            namespace='http://www.w3.org/2001/XMLSchema'),
        'saem.oai.seda.0.2-xsd',
    ),
    'seda02rng': (
        MetadataFormat(
            schema='http://relaxng.org/relaxng.xsd',
            namespace='http://www.w3.org/2001/XMLSchema'),
        'saem.oai.seda.0.2-rng',
    ),
    'seda1rng': (
        MetadataFormat(
            schema='http://relaxng.org/relaxng.xsd',
            namespace='http://www.w3.org/2001/XMLSchema'),
        'saem.oai.seda.1.0-rng',
    ),
    'seda2rng': (
        MetadataFormat(
            schema='http://relaxng.org/relaxng.xsd',
            namespace='http://www.w3.org/2001/XMLSchema'),
        'saem.oai.seda.2.0-rng',
    ),
}


class SEDAArchiveTransferOAIPMHRecordAdapter(oai.OAIPMHActiveRecordAdapter):
    """OAI-PMH adapter for SEDAArchiveTransfer entity type."""
    __select__ = oai.OAIPMHActiveRecordAdapter.__select__ & is_instance('SEDAArchiveTransfer')
    metadata_formats = PROFILE_METADATA_FORMATS.copy()

    @classmethod
    def set_definition(cls):
        specifier = oai.PublicETypeOAISetSpec(
            'SEDAArchiveTransfer', cls.identifier_attribute, 'profile')
        specifier['transferring_agent'] = TransferringAgentOAISetSpec()
        return specifier


class DeletedSEDAArchiveTransferOAIPMHRecordAdapter(oai.OAIPMHDeletedRecordAdapter):
    """OAI-PMH adapter for deleted SEDAArchiveTransfer."""
    __select__ = oai.OAIPMHDeletedRecordAdapter.__select__ & is_instance('SEDAArchiveTransfer')
    metadata_formats = PROFILE_METADATA_FORMATS.copy()


class OAISEDAExportView(export.SEDAExportView):
    __abstract__ = True

    def call(self):
        version, format = self.__regid__.replace('saem.oai.seda.', '').split('-')
        self._cw.view('seda.export', rset=self.cw_rset, version=version, format=format, w=self.w)


class OAISEDA02XSDExportView(OAISEDAExportView):
    __regid__ = 'saem.oai.seda.0.2-xsd'


class OAISEDA02RNGExportView(OAISEDAExportView):
    __regid__ = 'saem.oai.seda.0.2-rng'


class OAISEDA1RNGExportView(OAISEDAExportView):
    __regid__ = 'saem.oai.seda.1.0-rng'


class OAISEDA2RNGExportView(OAISEDAExportView):
    __regid__ = 'saem.oai.seda.2.0-rng'


def registration_callback(vreg):
    vreg.register_and_replace(SAEMSEDAArchiveTransferIClonableAdapter,
                              SEDAArchiveTransferIClonableAdapter)
    vreg.register_all(globals().values(), __name__,
                      butclasses=(SAEMSEDAArchiveTransferIClonableAdapter,))



from functools import partial
from cubicweb_saem_ref import workflows

sql = partial(sql, ask_confirm=False)

for etype, cols in (('ProfileDocument', ('file_type_code',
                                         'document_type_code',
                                         'character_set_code')),
                    ('SEDAFileTypeCode', ('file_type_code_value',))):
    for col in cols:
        sql('ALTER TABLE cw_{0} DROP COLUMN IF EXISTS cw_{1}'.format(etype, col))

db_indices = dict((idx, table) for idx, table, idx_def in sql(
    'SELECT indexname, tablename, indexdef FROM pg_indexes')
                  if not idx.startswith('pg_'))
for index in db_indices:
    if index.startswith('cw_') and '_pkey' not in index and '_cwuri_' not in index:
        if '_key' in index:
            sql('ALTER TABLE %s DROP CONSTRAINT %s' % (db_indices[index], index))
        else:
            sql('DROP INDEX %s' % index)


rename_entity_type('Agent', 'AuthorityRecord')
drop_relation_definition('AuthorityRecord', 'phone_number', 'PhoneNumber')
rename_entity_type('Authority', 'Organization')
add_entity_type('OrganizationUnit')
ou_wf = workflows.define_publication_workflow(add_workflow, 'OrganizationUnit')
add_entity_type('Agent')
agent_wf = workflows.define_publication_workflow(add_workflow, 'Agent')

# AuthorityRecord.ark_naa
add_relation_definition('AuthorityRecord', 'ark_naa', 'ArkNameAssigningAuthority')
rql('SET X ark_naa NAA WHERE X authority A, A ark_naa NAA')
commit()

# Cleanup database against things forbidden since a few releases that may make
# the migration fail if the migrated database isn't clean:
# * remove deposit role if agent has not archival_agent
rql('DELETE X archival_role R WHERE R name "deposit", NOT X archival_agent Y')
# * remove contact point if contact agent is not of the 'person' kind
rql('DELETE X contact_point Y WHERE NOT (Y agent_kind K, K name "person")')
# * remove contact point if agent is not of the 'authority' kind
rql('DELETE X contact_point Y WHERE NOT (X agent_kind K, K name "authority")')
# * remove agent user if agent is not of the 'person' kind
rql('DELETE X agent_user Y WHERE NOT (X agent_kind K, K name "person")')
# * remove archival agent if one of the end of the relation is not of the 'authority' kind
rql('DELETE X archival_agent Y WHERE NOT (X agent_kind K, K name "authority")')
rql('DELETE X archival_agent Y WHERE NOT (X agent_kind K, K name "authority")')
# * remove related schemes if the agent is not of the 'authority' kind
rql('DELETE X related_concept_scheme Y WHERE NOT (X agent_kind K, K name "authority")')
# * remove used profils if the agent is not of the 'authority' kind
rql('DELETE Y seda_transferring_agent X WHERE NOT (X agent_kind K, K name "authority")')

commit()

create_entity = partial(create_entity, ask_confirm=False)

def ou_for(authority_record_eid, name, state, authority, _cache={}):
    try:
        return _cache[authority_record_eid]
    except:
        ou = create_entity('OrganizationUnit', name=name,
                           in_state=ou_wf.state_by_name(state),
                           authority=authority,
                           authority_record=authority_record_eid)
        rql('SET A archival_role R WHERE A eid %(a)s, AR archival_role R, AR eid %(ar)s',
            {'a': ou.eid, 'ar': authority_record_eid})
        _cache[authority_record_eid] = ou
        return ou

def agent_for(authority_record_eid, name, state, authority, _cache={}):
    try:
        return _cache[authority_record_eid]
    except:
        agent = create_entity('Agent', name=name,
                              in_state=agent_wf.state_by_name(state),
                              authority=authority,
                              authority_record=authority_record_eid)
        _cache[authority_record_eid] = agent
        return agent


# Agent.agent_user
for x, name, state, authority, user in rql(
        'Any X,XN,XSN,XA, U WHERE X is AuthorityRecord, '
        'X name XN, X in_state XS, XS name XSN, X authority XA, '
        'X agent_user U'):
    agent = agent_for(x, name, state, authority)
    agent.cw_set(agent_user=user)

# OrganizationUnit.contact_point
for x, xname, xstate, xauthority, y, yname, ystate, yauthority in rql(
        'Any X,XN,XSN,XA, Y,YN,YSN,YA WHERE X is AuthorityRecord, '
        'X name XN, X in_state XS, XS name XSN, X authority XA, '
        'Y name YN, Y in_state YS, YS name YSN, Y authority YA, '
        'X contact_point Y'):
    ou = ou_for(x, xname, xstate, xauthority)
    ou.cw_set(contact_point=agent_for(y, yname, ystate, yauthority))

# Authority.archival_agent
for xauthority, y, yname, ystate, yauthority in rql(
        'DISTINCT Any XA, Y,YN,YSN,YA WHERE X is AuthorityRecord, '
        'X name XN, X authority XA, '
        'Y name YN, Y in_state YS, YS name YSN, Y authority YA, '
        'X archival_agent Y'):
    authority = cnx.entity_from_eid(xauthority)
    archival_ou = ou_for(y, yname, ystate, yauthority)
    if authority.archival_unit:
        if authority.archival_unit[0].eid != archival_ou.eid:
            print((u'authority {} has inconsistent archival unit {} and {}'.format(
                authority.name, authority.archival_unit[0].name, archival_ou.name
            )).encode('ascii', errors='replace'))
    else:
        authority.cw_set(archival_unit=archival_ou)


# OrganizationUnit.related_concept_scheme
for x, name, state, authority, scheme in rql(
        'Any X,XN,XSN,XA, CS WHERE X is AuthorityRecord, '
        'X name XN, X in_state XS, XS name XSN, X authority XA, '
        'X related_concept_scheme CS'):
    ou = ou_for(x, name, state, authority)
    ou.cw_set(related_concept_scheme=scheme)

# SEDAProfile.seda_transferring_agent
for x, name, state, authority, profile in rql(
        'Any X,XN,XSN,XA, P WHERE X is AuthorityRecord, '
        'X name XN, X in_state XS, XS name XSN, X authority XA, '
        'P seda_transferring_agent X'):
    ou = ou_for(x, name, state, authority)
    ou.cw_set(reverse_seda_transferring_agent=profile)

commit()

drop_relation_definition('SEDAProfile', 'seda_transferring_agent', 'AuthorityRecord')
drop_relation_definition('AuthorityRecord', 'agent_user', 'CWUser')
drop_relation_definition('AuthorityRecord', 'related_concept_scheme', 'ConceptScheme')
drop_relation_definition('AuthorityRecord', 'authority', 'Organization')
drop_relation_definition('AuthorityRecord', 'archival_role', 'ArchivalRole')
for rtype in ('contact_point', 'archival_agent'):
    drop_relation_definition('AuthorityRecord', rtype, 'AuthorityRecord')



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref views related to cloning of compound tree.

Only used for SEDA entities for now.
"""

from cubicweb import tags, _
from cubicweb.predicates import has_related_entities, is_in_state
from cubicweb.web import component

from cubicweb_seda.views.simplified import simplified_afs, simplified_pvs
from cubicweb_seda.views.clone import SEDAArchiveTransferCloneAction


# hide default relation components for 'new_version_of' rtype
simplified_afs.tag_subject_of(('*', 'new_version_of', '*'), 'main', 'hidden')
simplified_afs.tag_object_of(('*', 'new_version_of', '*'), 'main', 'hidden')
simplified_pvs.tag_subject_of(('*', 'new_version_of', '*'), 'hidden')
simplified_pvs.tag_object_of(('*', 'new_version_of', '*'), 'hidden')


# life-cycle management ############################################################################

# Only show clone action for SEDA profiles if no clone has been created yet

class SAEMSEDAArchiveTransferCloneAction(SEDAArchiveTransferCloneAction):
    __select__ = (SEDAArchiveTransferCloneAction.__select__
                  & ~has_related_entities('new_version_of', 'object')
                  & is_in_state('published'))
    title = _('new version')


def workflow_state(entity):
    """Return the state of the given entity."""
    return entity.cw_adapt_to('IWorkflowable').state


class SEDAArchiveTransferRelatedVersionsComponent(component.EntityCtxComponent):
    """Output a box containing relevant SEDA Profiles related to the current one."""
    __regid__ = 'saem.seda.relatedprofiles'
    __select__ = (component.EntityCtxComponent.__select__ &
                  (has_related_entities('new_version_of', role='subject') |
                   has_related_entities('new_version_of', role='object')))
    context = 'incontext'
    order = 11
    title = _('Related versions')

    def predecessor(self):
        """Yield the profile of which the displayed profile is a new version (draft or published).
        """
        if (workflow_state(self.entity) in (u'published', u'draft') and
                self.entity.predecessor):
            yield self.entity.predecessor

    def current_version(self, state):
        """Yield the latest profile, either published or draft, that replaces the displayed one.

        ``state`` parameter must be either "draft" or "published".
        """
        assert state in (u'published', u'draft')
        successor = self.entity.successor
        while successor and workflow_state(successor) != state:
            successor = successor.successor
        if successor:
            yield successor

    def render_body(self, w):
        profile_state = workflow_state(self.entity)
        display_entities = []
        if profile_state == u'draft':
            display_entities += self.predecessor()
        elif profile_state == u'published':
            display_entities += self.current_version(u'draft')
            display_entities += self.predecessor()
        elif profile_state == u'deprecated':
            display_entities += self.current_version(u'published')
        if display_entities:
            w(u'<ul class="list-group">')
            for entity in display_entities:
                w(u'<li class="list-group-item">')
                w(tags.span(self._cw._(workflow_state(entity)), klass='badge'))
                entity.view('incontext', w=w)
                w(u'</li>')
            w(u'</ul>')


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (SAEMSEDAArchiveTransferCloneAction,))
    vreg.register_and_replace(SAEMSEDAArchiveTransferCloneAction, SEDAArchiveTransferCloneAction)



from cubicweb_saem_ref.hooks import set_ark_and_cwuri


def fix_organization_ark(cnx):
    with cnx.security_enabled(write=False):
        rset = cnx.execute('Any O WHERE O ark NULL, O is Organization')
        for org in rset.entities():
            if org.ark_naa:
                naa_what = org.ark_naa[0].what
            else:
                # set dumb naa what that will have to be fixed later by hand,
                # but at least it will appear in the UI
                naa_what = u'XXX'
            attrs = {}
            set_ark_and_cwuri(cnx, org.eid, attrs, naa_what=naa_what)
            org.cw_set(**attrs)
        cnx.commit()


fix_organization_ark(cnx)



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref RDF views"""

from cubicweb.predicates import adaptable
from cubicweb.web.views import VID_BY_MIMETYPE, rdf


VID_BY_MIMETYPE['application/rdf+xml'] = 'rdf'


class SAEMRDFView(rdf.RDFView):
    """Override cw's rdf view to dispatch to primary.rdf / list.rdf views, as implemented in the
    skos cube.
    """
    __select__ = adaptable('RDFPrimary')

    def call(self):
        vid = 'primary.rdf' if len(self.cw_rset) == 1 else 'list.rdf'
        self.wview(vid, self.cw_rset)


def registration_callback(vreg):
    vreg.register_and_replace(SAEMRDFView, rdf.RDFView)
